//Fisher's Scoring Method

#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <cmath>

using namespace std;

#include "fisher.h"

fisher::fisher(int n_o,
	       int n_a,
	       int n_b,
	       int n_ab)

//Initializations...

{
  no = n_o;
  na = n_a;
  nb = n_b;
  nab = n_ab;
  p = pow((na + no)/(na + nb + no + nab),0.5) - pow(no/(na + nb + no + nab),0.5);
  q = pow((nb + no)/(na + nb + no + nab),0.5) - pow(no/(na + nb + no + nab),0.5);
  a = 0;
  b = 0;
  c = 0;
  info = (double*) malloc(3 * sizeof(double));
  scvec = (double*) malloc(2 * sizeof(double));
  s1 = 0;
  s2 = 0;
}

//Destructor Function...

fisher::~fisher()
{
  //  cout << "Calling destructor." << endl;
  free(info);
  free(scvec);
}


//Iterative p and q values...

void fisher::summary()
{
  cout <<"p="<< p << "q="<< q << endl;
}

//Doing the Iteration...

double fisher::result()
{
  infos();
  score();
    a = info[0];
    b = info[1];
    c = info[2];
    s1 = scvec[0];
    s2 = scvec[1];
    p = p - 1/(a * b - c * c) * (b * s1 - c * s2);
    q = q - 1/(a * b - c * c) * (a * s2 - c * s1);
}

//Calculating Score Function...

void fisher::score()
{
 
  scvec[0] = (-2 * no/(1 - p - q) + na * (1/p - 1/(2 - p - 2 * q))
 - nb * (2/(2 - q - 2 * p)) + nab/p);
  scvec[1] = (-2 * no/(1 - p - q) + nb * (1/q - 1/(2 - q - 2 * p)) 
 - na * (2/(2 - p - 2 * q)) + nab/q);
  return ;
}

//Calculating Information Matrix

void fisher::infos()
{
info[0] = (-2* no/pow((1 - p - q),2.0) - na * (1/(p *p) + 1/pow((2 - p 
- 2 * q),2.0)) - nb * 2/pow((2 - q - 2 * p),2.0) - nab/(p*p));

info[1] = (-2* no/pow((1 - p - q),2.0) - nb * (1/(q * q) + 1/pow((2 - q 
- 2 * p),2.0)) - na * 2/pow((2 - p - 2 * q),2.0) - nab/(q*q)); 

info[2] = (-2 * no/pow((1 - p - q),2.0) - na * 2/pow((2 - p - 2 * q),2) 
- nb * 4/pow((2 - q - 2 * p),2.0));
}

//Main Function...

int main() 
{
  fisher *f;
  int no, na, nb, nab;

  cout << "Enter the Number of persons with blood group O: ";
  cin >> no;
  cout <<"Enter the Number of persons with blood group A: ";
  cin >> na;
  cout << "Enter the Number of persons with blood group B: ";
  cin >> nb;
  cout << "Enter the Number of persons with blood group AB: ";
  cin >> nab;

  f = new fisher(no, na, nb, nab);

  for (int i = 0; i < 10; i++) {
    f->result();
    f->summary();
  }
  delete f;
}
